@@ -18,27 +18,44 @@ static int ifcore_search(lua_State *lua);
 static int ifcore_list(lua_State *lua);
 static int ifcore_lsub(lua_State *lua);
 static int ifcore_fetchfast(lua_State *lua);
+static int ifcore_fetchflags(lua_State *lua);
+static int ifcore_fetchdate(lua_State *lua);
+static int ifcore_fetchsize(lua_State *lua);
 static int ifcore_fetchheader(lua_State *lua);
 static int ifcore_fetchtext(lua_State *lua);
 static int ifcore_fetchfields(lua_State *lua);
 static int ifcore_store(lua_State *lua);
 static int ifcore_copy(lua_State *lua);
 static int ifcore_append(lua_State *lua);
+static int ifcore_create(lua_State *lua);
+static int ifcore_delete(lua_State *lua);
+static int ifcore_rename(lua_State *lua);
+static int ifcore_subscribe(lua_State *lua);
+static int ifcore_unsubscribe(lua_State *lua);
 
 
 /* Lua imapfilter core library functions. */
 static const luaL_reg ifcorelib[] = {
 	{ "noop", ifcore_noop },
-	{ "login", ifcore_login },
 	{ "logout", ifcore_logout },
-	{ "status", ifcore_status },
+	{ "login", ifcore_login },
 	{ "select", ifcore_select },
-	{ "close", ifcore_close },
-	{ "expunge", ifcore_expunge },
+	{ "create", ifcore_create },
+	{ "delete", ifcore_delete },
+	{ "rename", ifcore_rename },
+	{ "subscribe", ifcore_subscribe },
+	{ "unsubscribe", ifcore_unsubscribe },
 	{ "list", ifcore_list },
 	{ "lsub", ifcore_lsub },
+	{ "status", ifcore_status },
+	{ "append", ifcore_append },
+	{ "close", ifcore_close },
+	{ "expunge", ifcore_expunge },
 	{ "search", ifcore_search },
 	{ "fetchfast", ifcore_fetchfast },
+	{ "fetchflags", ifcore_fetchflags },
+	{ "fetchdate", ifcore_fetchdate },
+	{ "fetchsize", ifcore_fetchsize },
 
 	/*
 	 * RFC 822: message == header + body
@@ -53,7 +70,6 @@ static const luaL_reg ifcorelib[] = {
 	{ "fetchfields", ifcore_fetchfields },
 	{ "store", ifcore_store },
 	{ "copy", ifcore_copy },
-	{ "append", ifcore_append },
 	{ NULL, NULL }
 };
 
@@ -392,11 +408,12 @@ ifcore_search(lua_State *lua)
 
 	mesgs = NULL;
 
-	if (lua_gettop(lua) != 2)
+	if (lua_gettop(lua) != 3)
 		luaL_error(lua, "wrong number of arguments");
 
 	luaL_checktype(lua, 1, LUA_TTABLE);
 	luaL_checktype(lua, 2, LUA_TSTRING);
+	luaL_checktype(lua, 3, LUA_TSTRING);
 
 	lua_pushvalue(lua, 1);
 	if (!(s = get_table_string("server")))
@@ -406,9 +423,10 @@ ifcore_search(lua_State *lua)
 	p = DISCOVER_PORT(get_table_string("port"), get_table_string("ssl"));
 	lua_pop(lua, 1);
 
-	r = request_search(s, p, u, lua_tostring(lua, 2), &mesgs);
+	r = request_search(s, p, u, lua_tostring(lua, 2), lua_tostring(lua, 3),
+	    &mesgs);
 
-	lua_pop(lua, 2);
+	lua_pop(lua, 3);
 
 	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));
 
@@ -471,6 +489,135 @@ ifcore_fetchfast(lua_State *lua)
 }
 
 
+/*
+ * Core function to fetch message flags.
+ */
+static int
+ifcore_fetchflags(lua_State *lua)
+{
+	const char *s, *u, *p;
+	int r;
+	char *flags;
+
+	flags = NULL;
+
+	if (lua_gettop(lua) != 2)
+		luaL_error(lua, "wrong number of arguments");
+
+	luaL_checktype(lua, 1, LUA_TTABLE);
+	luaL_checktype(lua, 2, LUA_TSTRING);
+
+	lua_pushvalue(lua, 1);
+	if (!(s = get_table_string("server")))
+		luaL_error(lua, "no mail server specified");
+	if (!(u = get_table_string("username")))
+		luaL_error(lua, "no username specified");
+	p = DISCOVER_PORT(get_table_string("port"), get_table_string("ssl"));
+	lua_pop(lua, 1);
+
+	r = request_fetchflags(s, p, u, lua_tostring(lua, 2), &flags);
+
+	lua_pop(lua, 2);
+
+	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));
+
+	if (!flags)
+		return 1;
+
+	lua_pushstring(lua, flags);
+
+	xfree(flags);
+
+	return 2;
+}
+
+
+/*
+ * Core function to fetch message date.
+ */
+static int
+ifcore_fetchdate(lua_State *lua)
+{
+	const char *s, *u, *p;
+	int r;
+	char *date;
+
+	date = NULL;
+
+	if (lua_gettop(lua) != 2)
+		luaL_error(lua, "wrong number of arguments");
+
+	luaL_checktype(lua, 1, LUA_TTABLE);
+	luaL_checktype(lua, 2, LUA_TSTRING);
+
+	lua_pushvalue(lua, 1);
+	if (!(s = get_table_string("server")))
+		luaL_error(lua, "no mail server specified");
+	if (!(u = get_table_string("username")))
+		luaL_error(lua, "no username specified");
+	p = DISCOVER_PORT(get_table_string("port"), get_table_string("ssl"));
+	lua_pop(lua, 1);
+
+	r = request_fetchdate(s, p, u, lua_tostring(lua, 2), &date);
+
+	lua_pop(lua, 2);
+
+	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));
+
+	if (!date)
+		return 1;
+
+	lua_pushstring(lua, date);
+
+	xfree(date);
+
+	return 2;
+}
+
+
+/*
+ * Core function to fetch message size.
+ */
+static int
+ifcore_fetchsize(lua_State *lua)
+{
+	const char *s, *u, *p;
+	int r;
+	char *size;
+
+	size = NULL;
+
+	if (lua_gettop(lua) != 2)
+		luaL_error(lua, "wrong number of arguments");
+
+	luaL_checktype(lua, 1, LUA_TTABLE);
+	luaL_checktype(lua, 2, LUA_TSTRING);
+
+	lua_pushvalue(lua, 1);
+	if (!(s = get_table_string("server")))
+		luaL_error(lua, "no mail server specified");
+	if (!(u = get_table_string("username")))
+		luaL_error(lua, "no username specified");
+	p = DISCOVER_PORT(get_table_string("port"), get_table_string("ssl"));
+	lua_pop(lua, 1);
+
+	r = request_fetchsize(s, p, u, lua_tostring(lua, 2), &size);
+
+	lua_pop(lua, 2);
+
+	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));
+
+	if (!size)
+		return 1;
+
+	lua_pushstring(lua, size);
+
+	xfree(size);
+
+	return 2;
+}
+
+
 /*
  * Core function to fetch message header.
  */
@@ -717,6 +864,172 @@ ifcore_append(lua_State *lua)
 }
 
 
+/*
+ * Core function to create a mailbox.
+ */
+static int
+ifcore_create(lua_State *lua)
+{
+	const char *s, *u, *p;
+	int r;
+
+	if (lua_gettop(lua) != 2)
+		luaL_error(lua, "wrong number of arguments");
+
+	luaL_checktype(lua, 1, LUA_TTABLE);
+	luaL_checktype(lua, 2, LUA_TSTRING);
+
+	lua_pushvalue(lua, 1);
+	if (!(s = get_table_string("server")))
+		luaL_error(lua, "no mail server specified");
+	if (!(u = get_table_string("username")))
+		luaL_error(lua, "no username specified");
+	p = DISCOVER_PORT(get_table_string("port"), get_table_string("ssl"));
+	lua_pop(lua, 1);
+
+	r = request_create(s, p, u, lua_tostring(lua, 2));
+
+	lua_pop(lua, 2);
+
+	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));
+
+	return 1;
+}
+
+
+/*
+ * Core function to delete a mailbox.
+ */
+static int
+ifcore_delete(lua_State *lua)
+{
+	const char *s, *u, *p;
+	int r;
+
+	if (lua_gettop(lua) != 2)
+		luaL_error(lua, "wrong number of arguments");
+
+	luaL_checktype(lua, 1, LUA_TTABLE);
+	luaL_checktype(lua, 2, LUA_TSTRING);
+
+	lua_pushvalue(lua, 1);
+	if (!(s = get_table_string("server")))
+		luaL_error(lua, "no mail server specified");
+	if (!(u = get_table_string("username")))
+		luaL_error(lua, "no username specified");
+	p = DISCOVER_PORT(get_table_string("port"), get_table_string("ssl"));
+	lua_pop(lua, 1);
+
+	r = request_delete(s, p, u, lua_tostring(lua, 2));
+
+	lua_pop(lua, 2);
+
+	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));
+
+	return 1;
+}
+
+
+/*
+ * Core function to rename a mailbox.
+ */
+static int
+ifcore_rename(lua_State *lua)
+{
+	const char *s, *u, *p;
+	int r;
+
+	if (lua_gettop(lua) != 3)
+		luaL_error(lua, "wrong number of arguments");
+
+	luaL_checktype(lua, 1, LUA_TTABLE);
+	luaL_checktype(lua, 2, LUA_TSTRING);
+	luaL_checktype(lua, 3, LUA_TSTRING);
+
+	lua_pushvalue(lua, 1);
+	if (!(s = get_table_string("server")))
+		luaL_error(lua, "no mail server specified");
+	if (!(u = get_table_string("username")))
+		luaL_error(lua, "no username specified");
+	p = DISCOVER_PORT(get_table_string("port"), get_table_string("ssl"));
+	lua_pop(lua, 1);
+
+	r = request_rename(s, p, u, lua_tostring(lua, 2), lua_tostring(lua, 3));
+
+	lua_pop(lua, 3);
+
+	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));
+
+	return 1;
+}
+
+
+/*
+ * Core function to subscribe a mailbox.
+ */
+static int
+ifcore_subscribe(lua_State *lua)
+{
+	const char *s, *u, *p;
+	int r;
+
+	if (lua_gettop(lua) != 2)
+		luaL_error(lua, "wrong number of arguments");
+
+	luaL_checktype(lua, 1, LUA_TTABLE);
+	luaL_checktype(lua, 2, LUA_TSTRING);
+
+	lua_pushvalue(lua, 1);
+	if (!(s = get_table_string("server")))
+		luaL_error(lua, "no mail server specified");
+	if (!(u = get_table_string("username")))
+		luaL_error(lua, "no username specified");
+	p = DISCOVER_PORT(get_table_string("port"), get_table_string("ssl"));
+	lua_pop(lua, 1);
+
+	r = request_subscribe(s, p, u, lua_tostring(lua, 2));
+
+	lua_pop(lua, 2);
+
+	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));
+
+	return 1;
+}
+
+
+/*
+ * Core function to unsubscribe a mailbox.
+ */
+static int
+ifcore_unsubscribe(lua_State *lua)
+{
+	const char *s, *u, *p;
+	int r;
+
+	if (lua_gettop(lua) != 2)
+		luaL_error(lua, "wrong number of arguments");
+
+	luaL_checktype(lua, 1, LUA_TTABLE);
+	luaL_checktype(lua, 2, LUA_TSTRING);
+
+	lua_pushvalue(lua, 1);
+	if (!(s = get_table_string("server")))
+		luaL_error(lua, "no mail server specified");
+	if (!(u = get_table_string("username")))
+		luaL_error(lua, "no username specified");
+	p = DISCOVER_PORT(get_table_string("port"), get_table_string("ssl"));
+	lua_pop(lua, 1);
+
+	r = request_unsubscribe(s, p, u, lua_tostring(lua, 2));
+
+	lua_pop(lua, 2);
+
+	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));
+
+	return 1;
+}
+
+
 /*
  * Open imapfilter core library.
  */
