@@ -22,37 +22,49 @@ void interactive_mode(void);
 
 /*
  * Start the Lua interpreter, export IMAP core and system functions, load the
- * Lua interface and auxiliary functions, load and execute imapfilter's
- * configuration file.
+ * Lua interface functions, load and execute imapfilter's configuration file.
  */
 void
 start_lua()
 {
 
-#if LUA_VERSION_NUM >= 501
 	lua = luaL_newstate();
 
 	luaL_openlibs(lua);
-#else
-	lua = lua_open();
-
-	luaopen_base(lua);
-	luaopen_table(lua);
-	luaopen_io(lua);
-	luaopen_string(lua);
-	luaopen_math(lua);
-	luaopen_debug(lua);
-	luaopen_loadlib(lua);
-#endif
 
 	luaopen_ifcore(lua);
 	luaopen_ifsys(lua);
+	luaopen_ifre(lua);
 
 	lua_settop(lua, 0);
 
 	init_options();
 
-	if (luaL_loadfile(lua, PATHNAME_INTERFACE) ||
+	if (luaL_loadfile(lua, PATHNAME_COMMON) ||
+	    lua_pcall(lua, 0, LUA_MULTRET, 0))
+		fatal(ERROR_CONFIG, "%s\n", lua_tostring(lua, -1));
+
+	if (luaL_loadfile(lua, PATHNAME_SET) ||
+	    lua_pcall(lua, 0, LUA_MULTRET, 0))
+		fatal(ERROR_CONFIG, "%s\n", lua_tostring(lua, -1));
+
+	if (luaL_loadfile(lua, PATHNAME_REGEX) ||
+	    lua_pcall(lua, 0, LUA_MULTRET, 0))
+		fatal(ERROR_CONFIG, "%s\n", lua_tostring(lua, -1));
+
+	if (luaL_loadfile(lua, PATHNAME_ACCOUNT) ||
+	    lua_pcall(lua, 0, LUA_MULTRET, 0))
+		fatal(ERROR_CONFIG, "%s\n", lua_tostring(lua, -1));
+
+	if (luaL_loadfile(lua, PATHNAME_MAILBOX) ||
+	    lua_pcall(lua, 0, LUA_MULTRET, 0))
+		fatal(ERROR_CONFIG, "%s\n", lua_tostring(lua, -1));
+
+	if (luaL_loadfile(lua, PATHNAME_MESSAGE) ||
+	    lua_pcall(lua, 0, LUA_MULTRET, 0))
+		fatal(ERROR_CONFIG, "%s\n", lua_tostring(lua, -1));
+
+	if (luaL_loadfile(lua, PATHNAME_OPTIONS) ||
 	    lua_pcall(lua, 0, LUA_MULTRET, 0))
 		fatal(ERROR_CONFIG, "%s\n", lua_tostring(lua, -1));
 
@@ -60,6 +72,10 @@ start_lua()
 	    lua_pcall(lua, 0, LUA_MULTRET, 0))
 		fatal(ERROR_CONFIG, "%s\n", lua_tostring(lua, -1));
 
+	if (luaL_loadfile(lua, PATHNAME_DEPRECATED) ||
+	    lua_pcall(lua, 0, LUA_MULTRET, 0))
+		fatal(ERROR_CONFIG, "%s\n", lua_tostring(lua, -1));
+
 	if (opts.oneline != NULL) {
 		if (luaL_loadbuffer(lua, opts.oneline, strlen(opts.oneline),
 		    "=<command line>") || lua_pcall(lua, 0, LUA_MULTRET, 0))
@@ -97,13 +113,11 @@ init_options(void)
 
 	set_table_boolean("create", 0);
 	set_table_boolean("crammd5", 1);
-	set_table_boolean("expunge", 0);
+	set_table_boolean("expunge", 1);
 	set_table_boolean("namespace", 1);
 	set_table_boolean("starttls", 1);
 	set_table_boolean("subscribe", 0);
-	set_table_boolean("uid", 1);
 	set_table_number("timeout", 0);
-	set_table_string("charset", "");
 
 	lua_setglobal(lua, "options");
 }
