@@ -35,6 +35,7 @@ session_init(session *ssn)
 {
 
 	ssn->server = NULL;
+	ssn->port = NULL;
 	ssn->username = NULL;
 	ssn->socket = -1;
 #ifndef NO_SSLTLS
@@ -69,6 +70,8 @@ session_free(session *ssn)
 
 	if (ssn->server)
 		xfree(ssn->server);
+	if (ssn->port)
+		xfree(ssn->port);
 	if (ssn->username)
 		xfree(ssn->username);
 	if (ssn->ns.prefix)
@@ -81,15 +84,16 @@ session_free(session *ssn)
  * Based on the specified socket, find an active IMAP session.
  */
 session *
-session_find(const char *serv, const char *user)
+session_find(const char *serv, const char *port, const char *user)
 {
 	list *l;
 	session *s;
 
 	for (l = sessions; l; l = l->next) {
 		s = l->data;
-		if (!strncmp(s->server, serv, strlen(serv)) &&
-		    !strncmp(s->username, user, strlen(user)))
+		if (!strcmp(s->server, serv) &&
+		    !strcmp(s->port, port) &&
+		    !strcmp(s->username, user))
 			return s;
 	}
 
